var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import { generateApolloClient } from '@deep-foundation/hasura/client.js';
import Debug from 'debug';
import { DeepClient } from '../imports/client.js';
import { installPackage } from './1678940577209-deepcase.js';
import { containWithin, packageExists, sharePermissions } from './1664940577200-tsx.js';
const debug = Debug('deeplinks:migrations:jsonschema');
const log = debug.extend('log');
const error = debug.extend('error');
const rootClient = generateApolloClient({
    path: `${process.env.MIGRATIONS_HASURA_PATH}/v1/graphql`,
    ssl: !!+(process.env.MIGRATIONS_HASURA_SSL || 0),
    secret: process.env.MIGRATIONS_HASURA_SECRET,
});
const root = new DeepClient({
    apolloClient: rootClient,
});
const dc = '@deep-foundation/core';
const du = '@deep-foundation/unsafe';
export const up = () => __awaiter(void 0, void 0, void 0, function* () {
    log('up');
    const packageName = '@deep-foundation/jsonschema';
    if (!(yield packageExists(packageName))) {
        const adminId = yield root.id('deep', 'admin');
        const admin = yield root.login({ linkId: adminId });
        const deep = new DeepClient(Object.assign({ deep: root }, admin));
        yield installPackage(deep, '@deep-foundation/jsonschema');
        const packageId = yield root.id('@deep-foundation/jsonschema');
        yield sharePermissions(adminId, packageId);
        yield containWithin(adminId, packageId);
        yield deep.insert({
            in: {
                type_id: deep.idLocal(dc, 'Contain'),
                from_id: adminId,
            },
            type_id: deep.idLocal(dc, 'Rule'),
            out: { data: [
                    {
                        type_id: deep.idLocal(dc, 'RuleObject'),
                        to: {
                            type_id: deep.idLocal(dc, 'Selector'),
                            out: {
                                type_id: deep.idLocal(dc, 'SelectorInclude'),
                                to_id: yield deep.id(packageName),
                                out: {
                                    type_id: deep.idLocal(dc, 'SelectorTree'),
                                    to_id: deep.idLocal(dc, 'joinTree'),
                                },
                            },
                        },
                    },
                    {
                        type_id: deep.idLocal(dc, 'RuleSubject'),
                        to: {
                            type_id: deep.idLocal(dc, 'Selector'),
                            out: {
                                type_id: deep.idLocal(dc, 'SelectorInclude'),
                                to_id: yield deep.id(packageName),
                                out: {
                                    type_id: deep.idLocal(dc, 'SelectorTree'),
                                    to_id: deep.idLocal(dc, 'joinTree'),
                                },
                            },
                        },
                    },
                    {
                        type_id: deep.idLocal(dc, 'RuleAction'),
                        to: {
                            type_id: deep.idLocal(dc, 'Selector'),
                            out: {
                                type_id: deep.idLocal(dc, 'SelectorInclude'),
                                to_id: yield deep.id(du, 'AllowUnsafe'),
                                out: {
                                    type_id: deep.idLocal(dc, 'SelectorTree'),
                                    to_id: deep.idLocal(dc, 'typesTree'),
                                },
                            },
                        },
                    },
                ] },
        });
    }
});
export const down = () => __awaiter(void 0, void 0, void 0, function* () {
    log('down');
});
//# sourceMappingURL=1726413155878-jsonschema.js.map